//
//  Quad.swift
//  PictureEditor
//
//  Created by Tianna Henry-Lewis on 2023-04-18.
//

import Foundation
import MetalKit

struct Vertex {
  var position: simd_float3
  var textureCoord: simd_float2
}

struct Quad {
  let vertexBuffer: MTLBuffer
  let indexBuffer: MTLBuffer


  var vertices: [Vertex] = [
    Vertex(position: [-1, -1, 0], textureCoord: [0.0, 1.0]),
    Vertex(position: [ 1, -1, 0], textureCoord: [1.0, 1.0]),
    Vertex(position: [-1,  1, 0], textureCoord: [0.0, 0.0]),
    Vertex(position: [ 1,  1, 0], textureCoord: [1.0, 0.0])
  ]
  var indices: [UInt16] = [
    0, 3, 2,
    0, 1, 3
  ]

  init(device: MTLDevice, scale: Float = 1.0) {

    vertices = vertices.map {
      Vertex(position: $0.position * scale, textureCoord: $0.textureCoord)
    }

    guard let vertexBuffer = device.makeBuffer(
      bytes: &vertices,
      length: MemoryLayout<Vertex>.stride * vertices.count,
      options: []) else {
      fatalError("🛑 Quad Vertex Buffer Failed")
    }

    guard let indexBuffer = device.makeBuffer(
      bytes: &indices,
      length: MemoryLayout<UInt16>.stride * indices.count,
      options: []) else {
      fatalError("🛑 Quad Index Buffer Failed")
    }

    self.vertexBuffer = vertexBuffer
    self.indexBuffer = indexBuffer
  }
}
